//
//  HNModule.m
//  HinaCloudSDK
//
//  Created by dequal on 2023/5/30.
//

#import "HNModule.h"
#import <HinaCloudSDK/HinaCloudSDK.h>

@implementation HNModule


UNI_EXPORT_METHOD_SYNC(@selector(getDeviceUId))
- (NSString *)getDeviceUId {
    NSString *deviceUId = [HinaCloudSDK sharedInstance].deviceUId;
    return deviceUId;
}


UNI_EXPORT_METHOD_SYNC(@selector(init:))
- (void)init:(NSDictionary *)settings {
    if (![settings isKindOfClass:[NSDictionary class]]) {
        return;
    }
    NSString *serverURL = settings[@"serverUrl"];
    if (![serverURL isKindOfClass:[NSString class]]) {
        return;
    }
    
    HNBuildOptions *options = [[HNBuildOptions alloc] initWithServerURL:serverURL launchOptions:nil];
    NSNumber *enableLog = settings[@"enableLog"];
    if ([enableLog isKindOfClass:[NSNumber class]]) {
        options.enableLog = [enableLog boolValue];
    }
    NSNumber *autoTrack = settings[@"autoTrackTypePolicy"];
    if ([autoTrack isKindOfClass:[NSNumber class]]) {
        options.autoTrackEventType = [autoTrack integerValue];
    }
    NSNumber *networkTypePolicy = settings[@"networkTypePolicy"];
    if ([networkTypePolicy isKindOfClass:[NSNumber class]]) {
        options.autoTrackEventType = [networkTypePolicy integerValue];
    }
    NSNumber *flushInterval = settings[@"flushInterval"];
    if ([flushInterval isKindOfClass:[NSNumber class]]) {
        options.flushInterval = [flushInterval integerValue];
    }
    NSNumber *flushBulksize = settings[@"flushPendSize"];
    if ([flushBulksize isKindOfClass:[NSNumber class]]) {
        options.flushPendSize = [flushBulksize integerValue];
    }
    NSNumber *enableEncrypt = settings[@"encrypt"];
    if ([enableEncrypt isKindOfClass:[NSNumber class]]) {
        options.enableEncrypt = [enableEncrypt boolValue];
    }
    NSNumber *enableJavascriptBridge = settings[@"enableJSBridge"];
    if ([enableJavascriptBridge isKindOfClass:[NSNumber class]]) {
        //        options.enableJavaScriptBridge = [enableJavascriptBridge boolValue];
        options.enableJSBridge = [enableJavascriptBridge boolValue];
    }
    NSDictionary *iOSSettings = settings[@"ios"];
    if ([iOSSettings isKindOfClass:[NSDictionary class]] && [iOSSettings[@"max_cache_size"] isKindOfClass:[NSNumber class]]) {
        options.maxCacheSize = [iOSSettings[@"max_cache_size"] integerValue];
    }
    // add
    if ([settings isKindOfClass:[NSDictionary class]] && [settings[@"maxCacheSizeForIOS"] isKindOfClass:[NSNumber class]]) {
        options.maxCacheSize = [settings[@"maxCacheSizeForIOS"] integerValue];
    }
    // add
    if ([iOSSettings isKindOfClass:[NSDictionary class]] && [iOSSettings[@"maxCacheSizeForAndroid"] isKindOfClass:[NSNumber class]]) {
        
    }
    
    NSDictionary *visualizedSettings = settings[@"visualized"];
    if ([visualizedSettings isKindOfClass:[NSDictionary class]]) {
        if ([visualizedSettings[@"auto_track"] isKindOfClass:[NSNumber class]]) {
            options.enableVisualizedAutoTrack = [visualizedSettings[@"auto_track"] boolValue];
        }
        if ([visualizedSettings[@"properties"] isKindOfClass:[NSNumber class]]) {
            options.enableVisualizedProperties = [visualizedSettings[@"properties"] boolValue];
        }
    }
    NSNumber *enableHeatMap = settings[@"heat_map"];
    if ([enableHeatMap isKindOfClass:[NSNumber class]]) {
        options.enableHeatMap = [enableHeatMap boolValue];
    }
    
    dispatch_async(dispatch_get_main_queue(), ^{
        [HinaCloudSDK startWithConfigOptions:options];
    });
    
}

UNI_EXPORT_METHOD_SYNC(@selector(track:properties:))
- (void)track:(NSString *)eventName properties:(NSDictionary *)properties {
    if (![eventName isKindOfClass:NSString.class]) {
        NSLog(@" ❌ [HNModule Error] Event name[%@] not valid", eventName);
        return;
    }
    
    [[HinaCloudSDK sharedInstance] track:eventName withProperties:properties];
}

UNI_EXPORT_METHOD_SYNC(@selector(setDeviceUId:))
- (void)setDeviceUId:(NSString *)deviceUId {
    [[HinaCloudSDK sharedInstance] deviceUId:deviceUId];
}

UNI_EXPORT_METHOD_SYNC(@selector(userSet:))
- (void)userSet:(NSDictionary *)profileDict {
    @try {
        [[HinaCloudSDK sharedInstance] set:profileDict];
    } @catch (NSException *exception) {
        NSLog(@"[RNHinaCloud] error:%@",exception);
    }
}

UNI_EXPORT_METHOD_SYNC(@selector(userSetOnce:))
- (void)userSetOnce:(NSDictionary *)profileDict {
    @try {
        [[HinaCloudSDK sharedInstance] setOnce:profileDict];
    } @catch (NSException *exception) {
        NSLog(@"[RNHinaCloud] error:%@",exception);
    }
}

UNI_EXPORT_METHOD_SYNC(@selector(userAdd:))
- (void)userAdd:(NSDictionary *)profile{
    @try {
        [[HinaCloudSDK sharedInstance] add:profile];
    } @catch (NSException *exception) {
        NSLog(@"[RNHinaCloud] error:%@",exception);
    }
}

UNI_EXPORT_METHOD_SYNC(@selector(userUnset:))
- (void)userUnset:(NSString *) profile {
    @try {
        [[HinaCloudSDK sharedInstance] unset:profile];
    } @catch (NSException *exception) {
        NSLog(@"[RNHinaCloud] error:%@",exception);
    }
}

UNI_EXPORT_METHOD_SYNC(@selector(userDelete))
- (void)userDelete {
    @try {
        [[HinaCloudSDK sharedInstance] deleteUser];
    } @catch (NSException *exception) {
        NSLog(@"[RNHinaCloud] error:%@",exception);
    }
}

UNI_EXPORT_METHOD_SYNC(@selector(trackTimerStart:))
- (void)trackTimerStart:(NSString *)event {
    @try {
        [[HinaCloudSDK sharedInstance] trackTimerStart:event];
    } @catch (NSException *exception) {
        NSLog(@"[UniHinaCloud] error:%@",exception);
    }
}

UNI_EXPORT_METHOD_SYNC(@selector(trackTimerEnd:Properties:))
- (void)trackTimerEnd:(NSString *)event Properties:(NSDictionary *)propertyDict{
    @try {
        [[HinaCloudSDK sharedInstance] trackTimerEnd:event withProperties:propertyDict];
    } @catch (NSException *exception) {
        NSLog(@"[RNHinaCloud] error:%@",exception);
    }
}

UNI_EXPORT_METHOD_SYNC(@selector(registerCommonProperties:))
- (void)registerCommonProperties:(NSDictionary *)properties{
    @try {
        [[HinaCloudSDK sharedInstance] registerSuperProperties:properties];
    } @catch (NSException *exception) {
        NSLog(@"[RNHinaCloud] error:%@",exception);
    }
}

UNI_EXPORT_METHOD_SYNC(@selector(getPresetProperties))
- (NSDictionary *)getPresetProperties {
    @try {
        NSDictionary *presetProperties = [[HinaCloudSDK sharedInstance] getPresetProperties];
        return presetProperties;
    } @catch (NSException *exception) {
        NSLog(@"[RNHinaCloud] error:%@",exception);
    }
}

UNI_EXPORT_METHOD_SYNC(@selector(setFlushNetworkPolicy:))
- (void)setFlushNetworkPolicy:(NSInteger)networkType {
    @try {
        [[HinaCloudSDK sharedInstance] setFlushNetworkPolicy:networkType];
    } @catch (NSException *exception) {
        NSLog(@"[RNHinaCloud] error:%@",exception);
    }
}

UNI_EXPORT_METHOD_SYNC(@selector(flush))
- (void)flush {
    @try {
        [[HinaCloudSDK sharedInstance] flush];
    } @catch (NSException *exception) {
        NSLog(@"[RNHinaCloud] error:%@",exception);
    }
}

UNI_EXPORT_METHOD_SYNC(@selector(clear))
- (void)clear{
    @try {
        [[HinaCloudSDK sharedInstance] clear];
    } @catch (NSException *exception) {
        NSLog(@"[RNHinaCloud] error:%@",exception);
    }
}

UNI_EXPORT_METHOD_SYNC(@selector(setPushUId:pushId:))
- (void)setPushUId:(NSString *)pushkey pushId:(NSString *)pushId {
    @try {
        [[HinaCloudSDK sharedInstance] userPushKey:pushkey pushId:pushId];
    } @catch (NSException *exception) {
        NSLog(@"[RNHinaCloud] error:%@",exception);
    }
}

UNI_EXPORT_METHOD_SYNC(@selector(userAppend:profile:))
- (void)userAppend:(NSString *)user  profile:(NSArray *)profile{
    @try {
        NSSet *setCntent = [NSSet setWithArray:profile];
        [[HinaCloudSDK sharedInstance] append:user by:setCntent];
    } @catch (NSException *exception) {
        NSLog(@"[RNHinaCloud] error:%@",exception);
    }
}


UNI_EXPORT_METHOD_SYNC(@selector(setUserUId:))
- (void)setUserUId:(NSString *)loginId {
    @try {
        [[HinaCloudSDK sharedInstance] setUserUId:loginId];
    } @catch (NSException *exception) {
        NSLog(@"[RNHinaCloud] error:%@",exception);
    }
}

UNI_EXPORT_METHOD_SYNC(@selector(resetDeviceUid))
- (void)resetDeviceUid {
    @try {
        [[HinaCloudSDK sharedInstance] resetAnonymousId];
    } @catch (NSException *exception) {
        NSLog(@"[RNHinaCloud] error:%@",exception);
    }
}















//UNI_EXPORT_METHOD_SYNC(@selector(track:))
//- (void)track:(NSString *)event withProperties:(NSDictionary *)propertyDict {
//    @try {
//        [[HinaCloudSDK sharedInstance] track:event withProperties:propertyDict];
//    } @catch (NSException *exception) {
//        NSLog(@"[RNHinaCloud] error:%@",exception);
//    }
//}


//// 通过宏 UNI_EXPORT_METHOD 将异步方法暴露给 js 端
//UNI_EXPORT_METHOD(@selector(testAsyncFunc:callback:))
//
///// 异步方法（注：异步方法会在主线程（UI线程）执行）
///// @param options js 端调用方法时传递的参数
///// @param callback 回调方法，回传参数给 js 端
//- (void)testAsyncFunc:(NSDictionary *)options callback:(UniModuleKeepAliveCallback)callback {
//    // options 为 js 端调用此方法时传递的参数
//    NSLog(@"HNModule-%@",options);
//
//    // 可以在该方法中实现原生能力，然后通过 callback 回调到 js
//
//    // 回调方法，传递参数给 js 端 注：只支持返回 String 或 NSDictionary (map) 类型
//    if (callback) {
//        // 第一个参数为回传给js端的数据，第二个参数为标识，表示该回调方法是否支持多次调用，如果原生端需要多次回调js端则第二个参数传 YES;
//        callback(@"HNModule-success",NO);
//    }
//}
//
//// 通过宏 UNI_EXPORT_METHOD_SYNC 将同步方法暴露给 js 端
//UNI_EXPORT_METHOD_SYNC(@selector(testSyncFunc:))
//
///// 同步方法（注：同步方法会在 js 线程执行）
///// @param options js 端调用方法时传递的参数
//- (NSString *)testSyncFunc:(NSDictionary *)options {
//    // options 为 js 端调用此方法时传递的参数
//    NSLog(@"HNModule-%@",options);
//
//    /*
//     可以在该方法中实现原生功能，然后直接通过 return 返回参数给 js
//     */
//
//    // 同步返回参数给 js 端 注：只支持返回 String 或 NSDictionary (map) 类型
//    return @"HNModule-success";
//}

@end
